package cn.com.qmth.examcloud.core.reports.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.persistence.criteria.Predicate;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgsByIdListReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgsByIdListResp;
import cn.com.qmth.examcloud.core.reports.base.bean.StudentCountBean;
import cn.com.qmth.examcloud.core.reports.base.util.BatchGetDataUtil;
import cn.com.qmth.examcloud.core.reports.dao.StudentTotalCountRepo;
import cn.com.qmth.examcloud.core.reports.dao.entity.StudentTotalCountEntity;
import cn.com.qmth.examcloud.core.reports.service.StudentCountService;
import cn.com.qmth.examcloud.core.reports.service.StudentTotalCountService;

@Service
public class StudentTotalCountServiceImpl implements StudentTotalCountService {
	@Autowired
	private StudentTotalCountRepo studentTotalCountRepo;
	@Autowired
	private StudentCountService studentCountService;
	@Autowired
	private OrgCloudService orgCloudService;

	@Transactional
	@Override
	public synchronized void compute() {
		// TODO
	}

	@Override
	public Long getSumTotalCount() {
		return studentTotalCountRepo.getSumTotalCount();
	}

	@Override
	public PageInfo<StudentCountBean> queryPage(String rootOrgId, Integer pageNo, Integer pageSize) {
		Specification<StudentTotalCountEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (null != rootOrgId) {
				predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = PageRequest.of(pageNo - 1, pageSize, new Sort(Direction.DESC, "id"));

		Page<StudentTotalCountEntity> es = studentTotalCountRepo.findAll(specification, pageRequest);

		List<StudentCountBean> ret = Lists.newArrayList();

		for (StudentTotalCountEntity cur : es) {
			StudentCountBean bean = new StudentCountBean();
			bean.setTotalCount(cur.getTotalCount());
			bean.setRootOrgId(cur.getRootOrgId());
			ret.add(bean);
		}
		fillOnlineCount(ret);
		fillRootOrgName(ret);
		return new PageInfo<StudentCountBean>(es, ret);
	}

	private void fillOnlineCount(List<StudentCountBean> ret) {
		if (CollectionUtils.isEmpty(ret)) {
			return;
		}
		List<Long> temList = ret.stream().map(str -> str.getRootOrgId()).collect(Collectors.toList());
		List<StudentCountBean> sl = studentCountService.getCountByRootOrgIds(temList);
		if (CollectionUtils.isEmpty(sl)) {
			return;
		}
		Map<Long, Integer> map = sl.stream().collect(Collectors.toMap(StudentCountBean::getRootOrgId,
				account -> account.getOnlineCount(), (key1, key2) -> key2));
		for(StudentCountBean b:ret) {
			b.setOnlineCount(map.get(b.getRootOrgId()));
		}
	}

	private void fillRootOrgName(List<StudentCountBean> ret) {
		if (CollectionUtils.isEmpty(ret)) {
			return;
		}
		List<Long> param = ret.stream().map(str -> str.getRootOrgId()).collect(Collectors.toList());
		List<OrgBean> retList=new ArrayList<OrgBean>();
		GetOrgsByIdListReq req= new GetOrgsByIdListReq();
		BatchGetDataUtil<OrgBean, Long> tool = new BatchGetDataUtil<OrgBean, Long>() {
			@Override
			public List<OrgBean> getData(List<Long> paramList) {
				
				req.setOrgIdList(paramList);
				GetOrgsByIdListResp resp=orgCloudService.getOrgsByIdList(req);
				return resp.getOrgList();
			}

		};
		tool.getDataForBatch(retList, param, 100);
		if (CollectionUtils.isEmpty(retList)) {
			return;
		}
		Map<Long, String> map = retList.stream().collect(Collectors.toMap(OrgBean::getId,
				account -> account.getName(), (key1, key2) -> key2));
		for(StudentCountBean b:ret) {
			b.setRootOrgName(map.get(b.getRootOrgId()));
		}
	}
}
